@@ -8,6 +8,8 @@ from ..calling_conventions import SimRegArg, SimStackArg, SimCC, DefaultCC
 from ..sim_variable import SimStackVariable, SimRegisterVariable
 from ..knowledge_plugins.key_definitions.atoms import Register
 from ..knowledge_plugins.key_definitions.rd_model import ReachingDefinitionsModel
+from ..knowledge_plugins.variables.variable_access import VariableAccessSort
+from .reaching_definitions import get_all_definitions
 from . import Analysis, register_analysis
 
 if TYPE_CHECKING:
@@ -129,8 +131,7 @@ class CallingConventionAnalysis(Analysis):
         vm = self._variable_manager[self._function.addr]
 
         input_variables = vm.input_variables()
-
-        input_args = self._args_from_vars(input_variables)
+        input_args = self._args_from_vars(input_variables, vm)
 
         # TODO: properly determine sp_delta
         sp_delta = self.project.arch.bytes if self.project.arch.call_pushes_ret else 0
@@ -145,6 +146,9 @@ class CallingConventionAnalysis(Analysis):
             args = self._reorder_args(input_args, cc)
             cc.args = args
 
+            # set return value
+            cc.ret_val = cc.return_val
+
         return cc
 
     def _analyze_callsites(self) -> List[CallSiteFact]:  # pylint:disable=no-self-use
@@ -208,7 +212,7 @@ class CallingConventionAnalysis(Analysis):
         if default_cc_cls is not None:
 
             default_cc: SimCC = default_cc_cls(self.project.arch)
-            all_defs: Set['Definition'] = state.register_definitions.get_all_variables()
+            all_defs: Set['Definition'] = get_all_definitions(state.register_definitions)
 
             return_val = default_cc.RETURN_VAL
             if return_val is not None and isinstance(return_val, SimRegArg):
@@ -247,11 +251,12 @@ class CallingConventionAnalysis(Analysis):
 
         return cc
 
-    def _args_from_vars(self, variables):
+    def _args_from_vars(self, variables: List, var_manager):
         """
+        Derive function arguments from input variables.
 
-
-        :param list variables:
+        :param variables:
+        :param var_manager: The variable manager of this function.
         :return:
         """
 
@@ -261,6 +266,8 @@ class CallingConventionAnalysis(Analysis):
         else:
             ret_addr_offset = self.project.arch.bytes
 
+        reg_vars_with_single_access: List[SimRegisterVariable] = []
+
         for variable in variables:
             if isinstance(variable, SimStackVariable):
                 # a stack variable. convert it to a stack argument.
@@ -278,20 +285,55 @@ class CallingConventionAnalysis(Analysis):
                 reg_name = self.project.arch.translate_register_name(variable.reg, size=variable.size)
                 arg = SimRegArg(reg_name, variable.size)
                 args.add(arg)
+
+                accesses = var_manager.get_variable_accesses(variable)
+                if len(accesses) == 1:
+                    reg_vars_with_single_access.append(variable)
             else:
                 l.error('Unsupported type of variable %s.', type(variable))
 
-        return args
+        # the function might be saving registers at the beginning and restoring them at the end
+        # we should remove all registers that are strictly callee-saved and are not used anywhere in this function
+        end_blocks = [ (endpoint.addr, endpoint.size) for endpoint in self._function.endpoints_with_type['return'] ]
 
-    def _is_sane_register_variable(self, variable):
+        restored_reg_vars: Set[SimRegArg] = set()
+
+        # is there any instruction that restores this register in any end blocks?
+        if reg_vars_with_single_access:
+            if self._function.returning is False:
+                # no restoring is required if this function does not return
+                for var_ in reg_vars_with_single_access:
+                    reg_name = self.project.arch.translate_register_name(var_.reg, size=var_.size)
+                    restored_reg_vars.add(SimRegArg(reg_name, var_.size))
+
+            else:
+                reg_offsets: Set[int] = set(r.reg for r in reg_vars_with_single_access)
+                for var_ in var_manager.get_variables(sort="reg"):
+                    if var_.reg in reg_offsets:
+                        # check if there is only a write to it
+                        accesses = var_manager.get_variable_accesses(var_)
+                        if len(accesses) == 1 and accesses[0].access_type == VariableAccessSort.WRITE:
+                            found = False
+                            for end_block_addr, end_block_size in end_blocks:
+                                if end_block_addr <= accesses[0].location.ins_addr < end_block_addr + end_block_size:
+                                    found = True
+                                    break
+
+                            if found:
+                                reg_name = self.project.arch.translate_register_name(var_.reg, size=var_.size)
+                                restored_reg_vars.add(SimRegArg(reg_name, var_.size))
+
+        return args.difference(restored_reg_vars)
+
+    def _is_sane_register_variable(self, variable: SimRegisterVariable) -> bool:
         """
         Filters all registers that are surly not members of function arguments.
         This can be seen as a workaround, since VariableRecoveryFast sometimes gives input variables of cc_ndep (which
         is a VEX-specific register) :-(
 
-        :param SimRegisterVariable variable: The variable to test.
-        :return:                             True if it is an acceptable function argument, False otherwise.
-        :rtype:                              bool
+        :param variable: The variable to test.
+        :return:         True if it is an acceptable function argument, False otherwise.
+        :rtype:          bool
         """
 
         arch = self.project.arch
