@@ -1,4 +1,4 @@
-from typing import Optional, List, Tuple, Any, TYPE_CHECKING
+from typing import Optional, List, Tuple, Any, Set, TYPE_CHECKING
 import logging
 
 import archinfo
@@ -25,13 +25,15 @@ class CallSiteMaker(Analysis):
     """
     Add calling convention, declaration, and args to a call site.
     """
-    def __init__(self, block, reaching_definitions=None, stack_pointer_tracker=None):
+    def __init__(self, block, reaching_definitions=None, stack_pointer_tracker=None, ail_manager=None):
         self.block = block
 
         self._reaching_definitions = reaching_definitions
         self._stack_pointer_tracker = stack_pointer_tracker
+        self._ail_manager = ail_manager
 
         self.result_block = None
+        self.stack_arg_offsets: Optional[Set[Tuple[int,int]]] = None  # ins_addr, stack_offset
 
         self._analyze()
 
@@ -62,9 +64,11 @@ class CallSiteMaker(Analysis):
         args = [ ]
         arg_locs = None
 
+        stackarg_sp_diff = 0
         if func.calling_convention is None:
             l.warning('%s has an unknown calling convention.', repr(func))
         else:
+            stackarg_sp_diff = func.calling_convention.STACKARG_SP_DIFF
             if func.prototype is not None:
                 # Make arguments
                 arg_locs = func.calling_convention.arg_locs()
@@ -93,7 +97,7 @@ class CallSiteMaker(Analysis):
                         args.append(the_arg)
                     else:
                         # Reaching definitions are not available. Create a register expression instead.
-                        args.append(Expr.Register(None, None, offset, size * 8, reg_name=arg_loc.reg_name))
+                        args.append(Expr.Register(self._atom_idx(), None, offset, size * 8, reg_name=arg_loc.reg_name))
                 elif type(arg_loc) is SimStackArg:
 
                     stack_arg_locs.append(arg_loc)
@@ -137,44 +141,22 @@ class CallSiteMaker(Analysis):
                         # found it
                         new_stmts = new_stmts[:-1]
 
-
-        # remove statements that stores arguments on the stack
+        # calculate stack offsets for arguments that are put on the stack. these offsets will be consumed by
+        # simplification steps in the future, which may decide to remove statements that stores arguments on the stack.
         if stack_arg_locs:
             sp_offset = self._stack_pointer_tracker.offset_before(last_stmt.ins_addr, self.project.arch.sp_offset)
             if sp_offset is None:
                 l.warning("Failed to calculate the stack pointer offset at pc %#x. You may find redundant Store "
                           "statements.", last_stmt.ins_addr)
+                self.stack_arg_offsets = None
             else:
-                stack_arg_offsets = set((arg.stack_offset + sp_offset) for arg in stack_arg_locs)
-                old_stmts = new_stmts
-                new_stmts = [ ]
-                for stmt in old_stmts:
-                    if isinstance(stmt, Stmt.Store) and isinstance(stmt.addr, Expr.StackBaseOffset):
-                        offset = stmt.addr.offset
-                        if offset < 0:
-                            offset &= (1 << self.project.arch.bits) - 1
-                        if offset in stack_arg_offsets:
-                            continue
-                    new_stmts.append(stmt)
+                self.stack_arg_offsets = set(
+                    (last_stmt.ins_addr, sp_offset + arg.stack_offset - stackarg_sp_diff) for arg in stack_arg_locs
+                )
 
         ret_expr = last_stmt.ret_expr
-        if ret_expr is None:
-            ret_expr = None
-            if func.prototype is not None:
-                if func.prototype.returnty is not None and not isinstance(func.prototype.returnty, SimTypeBottom):
-                    # it has a return value
-                    if func.calling_convention is not None:
-                        ret_expr_size = func.prototype.returnty._with_arch(self.project.arch).size
-                        if ret_expr_size == 0:
-                            l.warning("Size of the return expression of function %r is 0, which is incorrect. "
-                                       "Force it to word size.", func)
-                            ret_expr_size = self.project.arch.bytes
-                        reg_offset = func.calling_convention.RETURN_VAL._fix_offset(
-                            None,
-                            ret_expr_size,
-                            arch=self.project.arch,
-                        )
-                        ret_expr = Expr.Register(None, None, reg_offset, ret_expr_size * 8)
+        # if ret_expr is None, it means in previous steps (such as during AIL simplification) we have deemed the return
+        # value of this call statement as useless and is removed.
 
         new_stmts.append(Stmt.Call(last_stmt, last_stmt.target,
                                    calling_convention=func.calling_convention,
@@ -261,9 +243,9 @@ class CallSiteMaker(Analysis):
 
         # TODO: Support extracting values
 
-        return None, Expr.Load(None,
-                         Expr.Register(None, None, self.project.arch.sp_offset, self.project.arch.bits) +
-                            Expr.Const(None, None, offset, self.project.arch.bits),
+        return None, Expr.Load(self._atom_idx(),
+                         Expr.Register(self._atom_idx(), None, self.project.arch.sp_offset, self.project.arch.bits) +
+                            Expr.Const(self._atom_idx(), None, offset, self.project.arch.bits),
                          size,
                          self.project.arch.memory_endness,
                          )
@@ -365,5 +347,8 @@ class CallSiteMaker(Analysis):
             return None
         return len(specifiers)
 
+    def _atom_idx(self) -> Optional[int]:
+        return self._ail_manager.next_atom() if self._ail_manager is not None else None
+
 
 register_analysis(CallSiteMaker, 'AILCallSiteMaker')
