@@ -94,6 +94,9 @@ class CConstruct:
 
                 # add (), {}, and [] to mapping for highlighting as well as the full functions name
                 elif isinstance(obj, (CClosingObject, CFunction)):
+                    if s is None:
+                        continue
+
                     pos_to_node.add_mapping(pos, len(s), obj)
 
                 if s.endswith('\n'):
@@ -270,9 +273,10 @@ class CFunction(CConstruct):  # pylint:disable=abstract-method
                 yield ", ", None
 
             variable = arg.unified_variable if arg.unified_variable is not None else arg.variable
-            raw_type_str = arg_type.c_repr(name=variable.name)
+            raw_type_str: str = arg_type.c_repr(name=variable.name)
             assert variable.name in raw_type_str
-            type_pre, type_post = raw_type_str.split(variable.name)
+            varname_pos = raw_type_str.rfind(variable.name)
+            type_pre, type_post = raw_type_str[:varname_pos], raw_type_str[varname_pos + len(variable.name):]
 
             yield type_pre, None
             yield variable.name, arg
@@ -1907,7 +1911,7 @@ class CStructuredCodeGenerator(BaseStructuredCodeGenerator, Analysis):
             return CReturn(None, tags=stmt.tags, codegen=self)
         elif len(stmt.ret_exprs) == 1:
             ret_expr = stmt.ret_exprs[0]
-            if ret_expr.variable is not None:
+            if not isinstance(ret_expr, Stmt.Call) and ret_expr.variable is not None:
                 return CReturn(self._cvariable(ret_expr.variable, offset=ret_expr.variable_offset),
                                tags=stmt.tags, codegen=self,
                                )
