@@ -90,7 +90,9 @@ class SimEngineVRAIL(
                 else:
                     l.debug("Unknown calling convention for function %s. Fall back to default calling convention.", target)
                     ret_expr: SimRegArg = self.project.factory.cc().RETURN_VAL
-                ret_reg_offset = self.project.arch.registers[ret_expr.reg_name][0]
+
+                if ret_expr is not None:
+                    ret_reg_offset = self.project.arch.registers[ret_expr.reg_name][0]
 
         # discover the prototype
         prototype: Optional[SimTypeFunction] = None
@@ -176,7 +178,7 @@ class SimEngineVRAIL(
             self._expr(expr.operands[0])
             self._expr(expr.operands[1])
             # still return a RichR instance
-            r = RichR(None)
+            r = RichR(self.state.top(expr.bits))
         return r
 
     def _ail_handle_Convert(self, expr: ailment.Expr.Convert):
@@ -191,6 +193,18 @@ class SimEngineVRAIL(
 
         return RichR(self.state.top(expr.to_bits), typevar=typevar)
 
+    def _ail_handle_Reinterpret(self, expr: ailment.Expr.Reinterpret):
+        r = self._expr(expr.operand)
+        typevar = None
+        if r.typevar is not None:
+            if isinstance(r.typevar, typevars.DerivedTypeVariable) and isinstance(r.typevar.label, typevars.ReinterpretAs):
+                # there is already a reinterpretas - overwrite it
+                typevar = typevars.DerivedTypeVariable(r.typevar.type_var, typevars.ReinterpretAs(expr.to_type, expr.to_bits))
+            else:
+                typevar = typevars.DerivedTypeVariable(r.typevar, typevars.ReinterpretAs(expr.to_type, expr.to_bits))
+
+        return RichR(self.state.top(expr.to_bits), typevar=typevar)
+
     def _ail_handle_StackBaseOffset(self, expr: ailment.Expr.StackBaseOffset):
         try:
             values: MultiValues = self.state.stack_region.load(self.state.stack_addr_from_offset(expr.offset),
@@ -234,11 +248,14 @@ class SimEngineVRAIL(
         r0 = self._expr(expr.iftrue)
         r1 = self._expr(expr.iffalse)
 
+        return RichR(self.state.top(expr.bits))
+
     def _ail_handle_Cmp(self, expr):  # pylint:disable=useless-return
         self._expr(expr.operands[0])
         self._expr(expr.operands[1])
         return RichR(self.state.top(1))
 
+    _ail_handle_CmpF = _ail_handle_Cmp
     _ail_handle_CmpEQ = _ail_handle_Cmp
     _ail_handle_CmpNE = _ail_handle_Cmp
     _ail_handle_CmpLT = _ail_handle_Cmp
@@ -312,6 +329,35 @@ class SimEngineVRAIL(
                      typevar=r0.typevar,
                      )
 
+    def _ail_handle_Mull(self, expr):
+
+        arg0, arg1 = expr.operands
+
+        r0 = self._expr(arg0)
+        r1 = self._expr(arg1)
+
+        if r0.data.concrete and r1.data.concrete:
+            # constants
+            result_size = expr.bits
+            if r0.data.size() < result_size:
+                if expr.signed:
+                    r0.data = claripy.SignExt(result_size - r0.data.size(), r0.data)
+                else:
+                    r0.data = claripy.ZeroExt(result_size - r0.data.size(), r0.data)
+            if r1.data.size() < result_size:
+                if expr.signed:
+                    r1.data = claripy.SignExt(result_size - r1.data.size(), r1.data)
+                else:
+                    r1.data = claripy.ZeroExt(result_size - r1.data.size(), r1.data)
+            return RichR(r0.data * r1.data,
+                         typevar=typeconsts.int_type(result_size),
+                         type_constraints=None)
+
+        r = self.state.top(expr.bits)
+        return RichR(r,
+                     typevar=r0.typevar,  # FIXME: the size is probably changed
+                     )
+
     def _ail_handle_Div(self, expr):
 
         arg0, arg1 = expr.operands
