@@ -1,5 +1,5 @@
 import weakref
-from typing import Optional, Iterable, Dict, Set, Generator, Tuple, TYPE_CHECKING
+from typing import Optional, Iterable, Dict, Set, Generator, Tuple, Union, TYPE_CHECKING
 import logging
 
 import claripy
@@ -81,13 +81,25 @@ class LiveDefinitions:
         self.track_tmps = track_tmps
         self._canonical_size: int = canonical_size  # TODO: Drop canonical_size
 
-        self.register_definitions = MultiValuedMemory(memory_id="reg", top_func=self.top, page_kwargs={'mo_cmp': self._mo_cmp}) \
+        self.register_definitions = MultiValuedMemory(memory_id="reg",
+                                                      top_func=self.top,
+                                                      skip_missing_values_during_merging=False,
+                                                      page_kwargs={'mo_cmp': self._mo_cmp}) \
             if register_definitions is None else register_definitions
-        self.stack_definitions = MultiValuedMemory(memory_id="mem", top_func=self.top, page_kwargs={'mo_cmp': self._mo_cmp}) \
+        self.stack_definitions = MultiValuedMemory(memory_id="mem",
+                                                   top_func=self.top,
+                                                   skip_missing_values_during_merging=False,
+                                                   page_kwargs={'mo_cmp': self._mo_cmp}) \
             if stack_definitions is None else stack_definitions
-        self.memory_definitions = MultiValuedMemory(memory_id="mem", top_func=self.top, page_kwargs={'mo_cmp': self._mo_cmp}) \
+        self.memory_definitions = MultiValuedMemory(memory_id="mem",
+                                                    top_func=self.top,
+                                                    skip_missing_values_during_merging=False,
+                                                    page_kwargs={'mo_cmp': self._mo_cmp}) \
             if memory_definitions is None else memory_definitions
-        self.heap_definitions = MultiValuedMemory(memory_id="mem", top_func=self.top, page_kwargs={'mo_cmp': self._mo_cmp}) \
+        self.heap_definitions = MultiValuedMemory(memory_id="mem",
+                                                  top_func=self.top,
+                                                  skip_missing_values_during_merging=False,
+                                                  page_kwargs={'mo_cmp': self._mo_cmp}) \
             if heap_definitions is None else heap_definitions
         self.tmps: Dict[int, Set[Definition]] = {}
 
@@ -131,21 +143,28 @@ class LiveDefinitions:
         return weakref.proxy(self)
 
     @staticmethod
-    def _mo_cmp(mo_self: Set['SimMemoryObject'], mo_other: Set['SimMemoryObject'], addr: int, size: int):  # pylint:disable=unused-argument
+    def _mo_cmp(mo_self: Union['SimMemoryObject', Set['SimMemoryObject']],
+                mo_other: Union['SimMemoryObject', Set['SimMemoryObject']], addr: int, size: int):  # pylint:disable=unused-argument
         # comparing bytes from two sets of memory objects
         # we don't need to resort to byte-level comparison. object-level is good enough.
 
-        if len(mo_self) == 1 and len(mo_other) == 1:
+        if type(mo_self) is set and type(mo_other) is set and len(mo_self) == 1 and len(mo_other) == 1:
             a = next(iter(mo_self))
             b = next(iter(mo_other))
             return a.object is b.object and a.endness == b.endness
 
         values_self = set()
         values_other = set()
-        for mo in mo_self:
-            values_self.add(mo.object)
-        for mo in mo_other:
-            values_other.add(mo.object)
+        if type(mo_self) is set:
+            for mo in mo_self:
+                values_self.add(mo.object)
+        else:
+            values_self.add(mo_self)
+        if type(mo_other) is set:
+            for mo in mo_other:
+                values_other.add(mo.object)
+        else:
+            values_other.add(mo_other)
         return values_self == values_other
 
     @staticmethod
@@ -408,7 +427,13 @@ class LiveDefinitions:
                     for v in vs:
                         yield from self.extract_defs(v)
             elif isinstance(atom.addr, HeapAddress):
-                yield self.heap_definitions.get_objects_by_offset(atom.addr.value)
+                try:
+                    mv: MultiValues = self.heap_definitions.load(atom.addr.value, size=atom.size, endness=atom.endness)
+                except SimMemoryMissingError:
+                    return
+                for vs in mv.values.values():
+                    for v in vs:
+                        yield from self.extract_defs(v)
             elif isinstance(atom.addr, int):
                 try:
                     values = self.memory_definitions.load(atom.addr, size=atom.size, endness=atom.endness)
@@ -439,14 +464,27 @@ class LiveDefinitions:
 
     def get_value_from_atom(self, atom: Atom) -> Optional[MultiValues]:
         if isinstance(atom, Register):
-            return self.register_definitions.load(atom.reg_offset, size=atom.size)
+            try:
+                return self.register_definitions.load(atom.reg_offset, size=atom.size)
+            except SimMemoryMissingError:
+                return None
         elif isinstance(atom, MemoryLocation):
             if isinstance(atom.addr, SpOffset):
-                return self.stack_definitions.load(atom.addr, size=atom.size)
+                stack_addr = self.stack_offset_to_stack_addr(atom.addr.offset)
+                try:
+                    return self.stack_definitions.load(stack_addr, size=atom.size, endness=atom.endness)
+                except SimMemoryMissingError:
+                    return None
             elif isinstance(atom.addr, HeapAddress):
-                return self.heap_definitions.load(atom.addr, size=atom.size)
+                try:
+                    return self.heap_definitions.load(atom.addr.value, size=atom.size, endness=atom.endness)
+                except SimMemoryMissingError:
+                    return None
             elif isinstance(atom.addr, int):
-                return self.memory_definitions.load(atom.addr, size=atom.size)
+                try:
+                    return self.memory_definitions.load(atom.addr, size=atom.size, endness=atom.endness)
+                except SimMemoryMissingError:
+                    return None
             else:
                 # ignore RegisterOffset
                 return None
