@@ -570,7 +570,7 @@ class PagedMemoryMixin(MemoryMixin):
 
 
 class LabeledPagesMixin(PagedMemoryMixin):
-    def load_with_labels(self, addr: int, size: int=None, endness=None, **kwargs) -> Tuple[claripy.ast.Base,Tuple[Tuple[int,int,Any]]]:
+    def load_with_labels(self, addr: int, size: int=None, endness=None, **kwargs) -> Tuple[claripy.ast.Base,Tuple[Tuple[int,int,int,Any]]]:
         if endness is None:
             endness = self.endness
 
@@ -613,6 +613,16 @@ class ListPagesMixin(PagedMemoryMixin):
 class MVListPagesMixin(PagedMemoryMixin):
     PAGE_TYPE = MVListPage
 
+    def __init__(self, *args, skip_missing_values_during_merging=False, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.skip_missing_values_during_merging = skip_missing_values_during_merging
+
+    @MemoryMixin.memo
+    def copy(self, memo) -> 'MVListPagesMixin':
+        r = super().copy(memo)
+        r.skip_missing_values_during_merging = self.skip_missing_values_during_merging
+        return r
+
 
 class ListPagesWithLabelsMixin(
     LabeledPagesMixin,
